# -*- coding: cp1252 -*-
#################################################################
#   Bipartite resiprokaltabeller                                #
#   Dette programmet konstruerer bipartite resiprokaltabeller   #
#   med j bit ut og j+2 bit inn                                 #
#                                                               #
#   format p inn data er:                                      #
#   1,b(1)b(2)b(3)...b(j+2)                                     #
#                                                               #
#   formatet op ut data er:                                     #
#   0,B(1)B(2)B(3)...B(j)                                       #
#   B(1) er altid lik 1                                         #
#   Det skal ogs vere med en ekstra ener p slutten i videre   #
#   beregninger                                                 #
#                                                               #
#   denne algoritmen benyttes for innbit strre enn 6           #
#################################################################

#importerer eksterne funksjoner.
#----------------------------------------------------------------
#import os
#import sys
import math

#lager P tabellen.
#----------------------------------------------------------------
def TabelP(k,u,j):
    i = 1
    for xh in range(2**(k+1)):       #gr gjennom alle blokkene  
        fs = firstspread(xh,k,u)
        ls = lastspread(xh,k,u)
        As = averagespread(fs,ls)

        for xm in range(2**(k+u)):   #gr gjennom alle segmenten
            sp = spread(xh,xm,k,u)
            ad = adjust(As,sp)
            p[xh][xm] = Pvalue(xh,xm,ad,k,u)
            pt = p[xh][xm]*2**(j+1)
            rd = rounddouwn(p[xh][xm],k,u)   #runner p ned til 3k+u+1 bit
            r[xh][xm] = float(rd)/float(2**(3*k+u+2))
            fi.write('      when "%s" =>' #%dec2bin(xh,k+1) + dec2bin(xm,k+u)
                     ' return "%s"; --%s\n' #%dec2bin(rd, j+2),xh*2**(k+u)+xm
                     %(str(dec2bin(xh, k+1)) + str(dec2bin(xm, k+u)),dec2bin(rd - 2**(3*k+u+1), j+2),xh*2**(k+u)+xm))
    return None

#lager N tabellen
#----------------------------------------------------------------
def TableN(k,u,j):
    i = 1
    for xh in range(2**(k+1)):
        for xl in range(2**k):
            fd = firstdiff(xh,xl,k,u)
            ld = lastdiff(xh,xl,k,u)
            n[xh][xl] = Nvalue(fd,ld)
            nt = n[xh][xl]*2**(j+1)
            rn = raundtonerest(n[xh][xl],k,u)  #runner av til nermeste (k+1bit)
            rnn[xh][xl] = float(rn)/float(2**(3*k+u+2))
            fi.write('      when "%s" =>' #%dec2bin(xh,k+1) + dec2bin(xl,k)
                     ' return "%s"; --%s\n' #%dec2bin(rn, j+2),xh*2**(k)+xl
                     %(str(dec2bin(xh, k+1)) + str(dec2bin(xl, k)),dec2bin(rn, k+1),xh*2**(k)+xl))
            #for  benytte 2's komplement. husk  bytte writesubtraction(...) til writeaddition(...)
            #fi.write('      when "%s" =>' #%dec2bin(xh,k+1) + dec2bin(xl,k)
            #         ' return "%s";\n' #%dec2bin(toskomplement(rn), j+2)
            #         %(str(dec2bin(xh, k+1)) + str(dec2bin(xl, k)),dec2bin(toskomplement(rn), k+1))
    return None    

#finner spredningen i frste segment for blokken xh
#----------------------------------------------------------------
#mid Resiprokalet av frste verdi i frste segment i blokk xh 
#minus mid resiprokalet til den siste verdien i samme segment og
#blokk
def firstspread(xh,k,u):
    return midResip(xh,0,0,k,u) - midResip(xh,0,2**k-1,k,u)

#finner spredningen i siste segment i blokk xh
#----------------------------------------------------------------
#mid Resiprokalet av frste verdi i siste segment i blokk xh 
#minus mid resiprokalet til den siste verdien i samme segment og
#blokk
def lastspread(xh,k,u):
    return midResip(xh,2**(k+u)-1,0,k,u) - midResip(xh,2**(k+u)-1,2**k-1,k,u)

#finner gjennomsnittelig spredningen i blokk xh
#----------------------------------------------------------------
#spredningen i frste segment pluss spredningen i siste segment i 
#blokk xh del p 2
def averagespread(fs,ls):
    return (fs+ls)/2

#finner spredningen i segment xm i blokk xh
#----------------------------------------------------------------
#mid Resiprokalet av frste verdi i segment xm i blokk xh 
#minus mid resiprokalet til den siste verdien i samme segment og
#blokk
def spread(xh,xm,k,u):
    return midResip(xh,xm,0,k,u) - midResip(xh,xm,2**k-1,k,u)

#finner gjennomsnittelig spredningen i blokk xh
#----------------------------------------------------------------
#spredningen i frste segment pluss spredningen i siste segment i 
#blokk xh del p 2
def adjust(As,sp):
    return (As-sp)/2

#finner P verdien til blokk xh segment xm.
#----------------------------------------------------------------
#regner ut mid resiprokalet til frste verdi i segment xm i blokk
#xh og justerer dette i forhold til gjennomsnittelig og egen
#spredning. (dette er "infinit riktige")
def Pvalue(xh,xm,ad,k,u):
    return midResip(xh,xm,0,k,u) + ad

#runder P ned og trunkerer den til 3k+u+1 bit
#----------------------------------------------------------------
def rounddouwn(p,k,u):
    return int(p*2**(3*k+u+2))

#Beregner resiprokalet til midt pungtet i intervallet.
#----------------------------------------------------------------
def midResip(xh,xm,xl,k,u):
    number = xh*2**(2*k+u)+xm*2**k+xl
    return 2**(3*k+u+1)/(2**(3*k+u+1)+number+0.5)

#Beregner differangsen til det frste mid resiprokalet i frste segment
#----------------------------------------------------------------
#mid resiprokalen frst frste segment i blokk xh minus mid  
#resiprokalet til verdi xl i samme blokk og segment
def firstdiff(xh,xl,k,u):
    return midResip(xh,0,0,k,u)-midResip(xh,0,xl,k,u)

#Beregner differangsen til det frst mid resiprokalet i siste segmentet
#----------------------------------------------------------------
#mid resiprokalen frst i siste segment i blokk xh minus mid  
#resiprokalet til verdi xl i samme blokk og segment
def lastdiff(xh,xl,k,u):
    return midResip(xh,2**(k+u)-1,0,k,u)-midResip(xh,2**(k+u)-1,xl,k,u)

#beregne den eksakte N verdien.
#----------------------------------------------------------------
#beregner gjennomsnittet av forsjellen til det frste mid 
#resiprokalet i frste og siste segment. med andre ord
#(fistdiff(xh,xl)-lastdiff(xh,xl))/2
def Nvalue(fd,ld):
    return (fd+ld)/2

#runner av til nrmeste tall.
#----------------------------------------------------------------
def raundtonerest(n,k,u):
    n = n*2**(3*k+u+2)
    if(n-int(n) > 0.5):
        return int(n+0.5)
    else: return int(n)

#ommgjr fra desimaltall til binre tall
#----------------------------------------------------------------
def dec2bin(d, antbit):
    bin = ''
    if d == 0: bin = '0'
    else:
        while(d > 0):
            bin = str(d%2) + bin #+ som string opperator ikke matematisk
            d = d >> 1
    for i in range(antbit - len(bin)):
        bin = '0' + bin
    return bin

#ommgjr til 2's komplement
#---------------------------------------------------------------
def toskomplement(orginal, k):
    return 2**(k+1)-orginal

#skriver til entety vhdl fila.
#---------------------------------------------------------------
def writeentetyfile(j):
    fe.write('-- File name :  newton_int_divider_bipartit_reci_rom_rtl.vhd\n'
             '-- Title     :  initiell reciprocal value\n'
             '-- Module    :  initvalue\n'
             '-- \n'
             '-- Purpose   :  create initial reciprocal value\n\n')

    fe.write('LIBRARY IEEE;\n'
             'USE IEEE.std_logic_1164.all;\n'
             'USE IEEE.std_logic_signed.all;\n\n')

    fe.write('entity newton_int_divider_bipartit_reci_rom is\n'
             '  port(\n'
             '       in1   :  in std_logic_vector(%s downto 0);\n' #%(j+1)
             '       out1  :  out std_logic_vector(%s downto 0)\n' #%(j+1)
             '	    );\n'
             'end newton_int_divider_bipartit_reci_rom;\n\n' %(j+1,j+1))
    return None

#skriver hedderen til vhdl rtl fila.
#----------------------------------------------------------------
def writehedderrtl():
    fi.write('-- File name :  newton_int_divider_bipartit_reci_rom_rtl.vhd\n'
             '-- Title     :  initiell reciprocal value\n'
             '-- Module    :  initvalue\n'
             '-- \n'
             '-- Purpose   :  create initial reciprocal value\n\n')

    fi.write('LIBRARY IEEE;\n'
             'USE IEEE.std_logic_1164.all;\n'
             'USE IEEE.std_logic_signed.all;\n\n')

    fi.write('architecture rtl of newton_int_divider_bipartit_reci_rom is\n')
    return None

#skriver P tabellen
#----------------------------------------------------------------
def writePtable(j,k,u):
    fi.write('  function look_up_rom_Ptable(address : std_logic_vector(%s downto 0)) return std_logic_vector is\n' #%2*k+u
             '  begin\n'
             '    case address is\n' %(2*k+u))
    TabelP(k,u,j)
    tilegg = '\"' + '0'*(j+1) + '\"' #lager riktig antall nuller for "when others"
    fi.write('      when others => return %s;\n'
             '    end case;\n'
             '  end look_up_rom_Ptable;\n\n' %tilegg)
    return None

#skriver N tabellen
#----------------------------------------------------------------
def writeNtable(k,u,j):
    fi.write('  function look_up_rom_Ntable(address : std_logic_vector(%s downto 0)) return std_logic_vector is\n' #%2*k
             '  begin\n'
             '    case address is\n' %(2*k))
    TableN(k,u,j)
    tilegg = '\"' + '0'*(k+1) + '\"' #lager riktig antall nuller for "when others"
    fi.write('      when others => return  %s;\n'
             '    end case;\n'
             '  end look_up_rom_Ntable;\n\n' %tilegg)
    return None

#lager sammenlegningen av tallene
#----------------------------------------------------------------
def writesubtraction(j,k,u):
    tillegg = '0'*(2*k+u+2) #legger til riktig antall nuller
    fi.write('  signal temp_out : std_logic_vector(%s downto 0);\n' #%j+2+2
             'begin\n'
             #'    out1 <= ("01" & look_up_rom_Ptable(in1(%s downto %s))) - ( "%s" & look_up_rom_Ntable(in1(%s downto %s) & in1(%s downto 0)));\n'
             '  temp_out <= ("01" & look_up_rom_Ptable(in1(%s downto %s)) & \'1\') - ( "%s" & look_up_rom_Ntable(in1(%s downto %s) & in1(%s downto 0)) & \'0\');\n'
             '  out1 <= temp_out(%s downto 3);\n' #%j+2+2
             'end rtl;\n\n' %(j+4, j+1, k, tillegg, j+1, 2*k+u, k-1, j+4))
    return None

#lager sammenlegningen av tallene med 2's komplement
#----------------------------------------------------------------
#def writeaddition(j,k,u):
#    tillegg = '1'*(2*k+u+2) #legger til riktig antall nuller
#    fi.write('begin\n'
#             '    out1 <= ("01" & look_up_rom_Ptable(in1(%s downto %s))) + ( "%s" & look_up_rom_Ntable(in1(%s downto %s) & in1(%s downto 0)));\n'
#             'end rtl;\n\n' %(j+1, k, tillegg, j+1, 2*k+u, k-1))
#    return None

#normaliserer dividenden
#----------------------------------------------------------------
def normelize(y):
    a = math.log(y,2)
    return y/2**(int(a))

#finner det korrekte resiprokalet(s korekt som den blir i python)
#----------------------------------------------------------------
def correctdiv(y, numbit):
    return 1/normelize(float(y))

#trekker ut xh av divisoren
#----------------------------------------------------------------
def findxhxmxl(y,numbit,k,u):
    y = normelize(float(y))
    print y
    temp = (y-1.0)*float(2**(k+1))
    #print temp
    xh = int(temp)
    print xh
    temp = (temp - float(xh))*float(2**(k+u))
    xm = int(temp)
    print xm
    temp = (temp - float(xm))*float(2**(k))
    xl = int(temp)
    print xl
    return xh, xm, xl

#finner resiprokalet ut fra de uavrunnede tabellene
#----------------------------------------------------------------
def appreci(y,k,u,numbit):
    xh, xm, xl = findxhxmxl(y,numbit,k,u)
    return p[xh][xm] - n[xh][xl]

#finner resiprokalet ut fra de avrunnede tabellene
#----------------------------------------------------------------
def apprecirouned(y,k,u,numbit):
    xh, xm, xl = findxhxmxl(y,numbit,k,u)
    return r[xh][xm] - rnn[xh][xl]

#beregner antall riktige bit
#----------------------------------------------------------------
def numcorrbit(diff):
    if(diff < 0):
        diff *= (-1.0)
    return math.log(diff,2)

#lager tabell pakken til oppfrsels modellen
#----------------------------------------------------------------
def writepck(numbit, j, k, u):
    #headder
    fp.write('library IEEE;\n'
             'use IEEE.std_logic_1164.all;\n'
             'use IEEE.numeric_std.all;\n'
             'use work.divlib.all;\n\n')

    #konstanter
    fp.write('package newton_reci_table is\n\n'
             '  constant q : natural := %s;\n' #antall bit totalt (numbit)
             '  constant k : natural := %s;\n' #antall riktige bit (j)
             '  constant g : natural := %s;\n' #antall garderingsbitt
             '  constant u : natural := %s;\n' #differangsen mellom xm og xl (u)
             '  constant m : natural := %s;\n\n' #samme som k her i skriptet
             %(numbit, j, 2, u, k))

    #deklarere typer og prosedyren
    fp.write('  subtype depthP is std_logic_vector(k+1 downto 0);\n' #j+2
             '  subtype depthN is std_logic_vector(m downto 0);\n\n' #k+1  
             '  type look_up_tableP is array(2**(2*m+u+1) to 2**(2*m+u+2)-1) of depthP;\n' #2*k+u,2*k+u
             '  type look_up_tableN is array(2**(2*m+u+1) to 2**(2*m+u+2)-1) of depthN;\n\n' #2*k+u,2*k+u
             '  procedure init_look_up_table(tableP : inout look_up_tableP; tableN : inout look_up_tableN);\n'
             )#%(j+2, k+1, 2*k+u, 2*k+u, 2*k+u, 2*k+u))

    #avslutter pakken
    fp.write('end newton_reci_table;')
    return None

#lager tabell pakke bodyen til oppfrsels modellen
#----------------------------------------------------------------
def writepckbdy(j, k, u):
    #hedder
    fb.write('library IEEE;\n'
             'use IEEE.std_logic_1164.all;\n'
             'use IEEE.numeric_std.all;\n'
             'use work.divlib.all;\n\n')

    fb.write('package body newton_reci_table is\n\n')

    fb.write('  procedure init_look_up_table(tableP : inout look_up_tableP; tableN : inout look_up_tableN) is\n'
             '    variable firstspread    : real;\n'
             '    variable lastspread     : real;\n'
             '    variable averagespread  : real;\n'
             '    variable spread         : real;\n'
             '    variable adjust         : real;\n'
             '    variable p              : real;\n'
             '    variable p_vec          : std_logic_vector(k+2 downto 0);\n' #j+2
             '    variable firstdiff      : real;\n'
             '    variable lastdiff       : real;\n'
             '    variable n              : real;\n'
             '    variable n_vec          : std_logic_vector(m downto 0);\n\n' # ommgjordt til full lengde. gammel =k+1
             )#%(j+2, k+1))

    fb.write('    function resipmid(xh : integer; xm : integer; xl : integer) return real is\n'
             '      variable temp : real;\n'
             '    begin\n'
             '      temp := real(xh)*real(2**(2*m+u))+real(xm)*real(2**m)+real(xl);\n'
             '      return real(2**(3*m+u+1))/(real(2**(3*m+u+1))+temp+0.5);\n'
             '    end resipmid;\n\n')

    fb.write('  begin\n'
             '    for ixh in 0 to 2**(m+1)-1 loop\n' #k
             '      firstspread := resipmid(ixh, 0, 0)-resipmid(ixh, 0, 2**(m)-1);\n' #k
             '      lastspread := resipmid(ixh, 2**(m+u)-1, 0)-resipmid(ixh, 2**(m+u)-1, 2**m-1);\n' #k,u,k,u,k
             '      averagespread := (firstspread + lastspread)/2.0;\n'
             '      for ixm in 0 to 2**(m+u)-1 loop\n' #k,u
             '        spread := resipmid(ixh, ixm,0)-resipmid(ixh,ixm,2**m-1);\n' #k
             '        adjust := (averagespread - spread)/2.0;\n'
             '        p := (resipmid(ixh,ixm,0)+adjust)*real(2**(3*m+u+2))-0.5;\n' #k,u
             '        p_vec := std_logic_vector(to_unsigned(integer(p),3*m+u+2));\n' #k,u
             '        tableP(ixh*2**(m+u)+ixm+2**(2*m+u+1)) := p_vec(3*m+u downto 0);\n' #k,u,k,u
             '      end loop;\n'
             '    end loop;\n\n'
             ''
             '    for jxh in 0 to 2**(m+1)-1 loop\n' #k
             '      for jxl in 0 to 2**(m)-1 loop\n' #k
             '        firstdiff := resipmid(jxh,0,0)-resipmid(jxh,0,jxl);\n'
             '        lastdiff := resipmid(jxh,2**(m+u)-1,0) - resipmid(jxh,2**(m+u)-1,jxl);\n' #k,u,k,u
             '        n := ((firstdiff+lastdiff)/2.0)*real(2**(3*m+u+2));\n'
             '        n_vec := std_logic_vector(to_unsigned((integer(n)), m+1));\n' #k
             '        tableN(jxh*2**(m)+jxl+2**(2*m+1)) := n_vec;\n' #k,k 
             '      end loop;\n'
             '    end loop;\n'
             )#ikke lenger riktig:%(k, k, k, u, k, u, k, k, u, k, k, u, k, u, k, u, k, k, k, u, k, u, k, k))

    fb.write('  end init_look_up_table;\n'
             'end newton_reci_table;')
    return None

#lager tabell pakken for oppfrsels modellen ved kvaderat beregninger
#----------------------------------------------------------------
def writepcksqr(numbit, j, k, u):
    #headder
    fsrp.write('library IEEE;\n'
             'use IEEE.std_logic_1164.all;\n'
             'use IEEE.numeric_std.all;\n'
             'use ieee.math_real.all;\n'
             'use work.divlib.all;\n\n')

    #konstanter
    fsrp.write('package newton_reci_table_sr is\n\n'
             '  constant q : natural := %s;\n' #antall bit totalt (numbit)
             '  constant k : natural := %s;\n' #antall riktige bit (j)
             '  constant g : natural := %s;\n' #antall garderingsbitt
             '  constant u : natural := %s;\n' #differangsen mellom xm og xl (u)
             '  constant m : natural := %s;\n\n' #samme som k her i skriptet
             %(numbit, j, 2, u, k))

    #deklarere typer og prosedyren
    fsrp.write('  subtype depthP is std_logic_vector(k+1 downto 0);\n' #j+2
             '  subtype depthN is std_logic_vector(m downto 0);\n\n' #k+1  
             '  type look_up_tableP is array(2**(2*m+u+1) to 2**(2*m+u+2)-1) of depthP;\n' #2*k+u,2*k+u
             '  type look_up_tableN is array(2**(2*m+u+1) to 2**(2*m+u+2)-1) of depthN;\n\n' #2*k+u,2*k+u
             '  procedure init_look_up_table(tableP : inout look_up_tableP; tableN : inout look_up_tableN);\n'
             )#%(j+2, k+1, 2*k+u, 2*k+u, 2*k+u, 2*k+u))

    #avslutter pakken
    fsrp.write('end newton_reci_table_sr;')
    return None

#lager tabell pakke bodyen til oppfrsels modellen
#----------------------------------------------------------------
def writepckbdysqr(j, k, u):
    #hedder
    fsrb.write('library IEEE;\n'
             'use IEEE.std_logic_1164.all;\n'
             'use IEEE.numeric_std.all;\n'
             'use ieee.math_real.all;\n'  
             'use work.divlib.all;\n\n')

    fsrb.write('package body newton_reci_table_sr is\n\n')

    fsrb.write('  procedure init_look_up_table(tableP : inout look_up_tableP; tableN : inout look_up_tableN) is\n'
             '    variable firstspread    : real;\n'
             '    variable lastspread     : real;\n'
             '    variable averagespread  : real;\n'
             '    variable spread         : real;\n'
             '    variable adjust         : real;\n'
             '    variable p              : real;\n'
             '    variable p_vec          : std_logic_vector(k+2 downto 0);\n' #j+2
             '    variable firstdiff      : real;\n'
             '    variable lastdiff       : real;\n'
             '    variable n              : real;\n'
             '    variable n_vec          : std_logic_vector(m downto 0);\n\n' # ommgjordt til full lengde. gammel =k+1
             )#%(j+2, k+1))

    fsrb.write('    function resipmid(xh : integer; xm : integer; xl : integer) return real is\n'
             '      variable temp1, temp2, temp3 : real;\n'
             '    begin\n'
             '      temp1 := real(xh)*real(2**(2*m+u))+real(xm)*real(2**m)+real(xl);\n'
             '      temp2 := sqrt(real(2**(3*m+u+1))+temp1);\n'
             '      temp3 := sqrt(real(2**(3*m+u+1))+temp1+1.0);\n'  
             '      return 2.0*sqrt(real(2**(3*m+u+1)))/(temp2+temp3);\n'
             '    end resipmid;\n\n')

    fsrb.write('  begin\n'
             '    for ixh in 0 to 2**(m+1)-1 loop\n' #k
             '      firstspread := resipmid(ixh, 0, 0)-resipmid(ixh, 0, 2**(m)-1);\n' #k
             '      lastspread := resipmid(ixh, 2**(m+u)-1, 0)-resipmid(ixh, 2**(m+u)-1, 2**m-1);\n' #k,u,k,u,k
             '      averagespread := (firstspread + lastspread)/2.0;\n'
             '      for ixm in 0 to 2**(m+u)-1 loop\n' #k,u
             '        spread := resipmid(ixh, ixm,0)-resipmid(ixh,ixm,2**m-1);\n' #k
             '        adjust := (averagespread - spread)/2.0;\n'
             '        p := (resipmid(ixh,ixm,0)+adjust)*real(2**(3*m+u+2))-0.5;\n' #k,u
             '        p_vec := std_logic_vector(to_unsigned(integer(p),3*m+u+2));\n' #k,u
             '        tableP(ixh*2**(m+u)+ixm+2**(2*m+u+1)) := p_vec(3*m+u downto 0);\n' #k,u,k,u
             '      end loop;\n'
             '    end loop;\n\n'
             ''
             '    for jxh in 0 to 2**(m+1)-1 loop\n' #k
             '      for jxl in 0 to 2**(m)-1 loop\n' #k
             '        firstdiff := resipmid(jxh,0,0)-resipmid(jxh,0,jxl);\n'
             '        lastdiff := resipmid(jxh,2**(m+u)-1,0) - resipmid(jxh,2**(m+u)-1,jxl);\n' #k,u,k,u
             '        n := ((firstdiff+lastdiff)/2.0)*real(2**(3*m+u+2));\n'
             '        n_vec := std_logic_vector(to_unsigned((integer(n)), m+1));\n' #k
             '        tableN(jxh*2**(m)+jxl+2**(2*m+1)) := n_vec;\n' #k,k 
             '      end loop;\n'
             '    end loop;\n'
             )

    fsrb.write('  end init_look_up_table;\n'
             'end newton_reci_table_sr;')
    return None

#kjrer newton raphson metoden
#----------------------------------------------------------------
def newtonrap(y, z, itt, antbit):
    for i in range(itt):
        print '%f\n%f\n%f\n' %(y, z, itt)
        z = z*(2-y*z)
        z = float(int(z*2**(antbit+2)))/2**(antbit+2)
    return z


#Main
#----------------------------------------------------------------
#hoved delen av programmet
print "starter"
bit = 16
j = 9
k = int((j+2)/3)
if((j+2)%3 == 0):
    u = -1
elif((j+2)%3 == 1):
    u = 0
else: u = 1

#lager variabler som skal brukes globalt
p = [float]*2**(k+1)
for l in range(2**(k+1)):
    p[l] = [float]*2**(k+u)
r = [float]*2**(k+1)
for l in range(2**(k+1)):
    r[l] = [float]*2**(k+u)
n = [float]*2**(k+1)
for l in range(2**(k+1)):
    n[l] = [float]*2**k
rnn = [float]*2**(k+1)
for l in range(2**(k+1)):
    rnn[l] = [float]*2**k

#pner fil for skriving
try:
    #bytt w med a for  legge til isteden for overskrivning
    fi = open('newton_int_divider_bipartit_reci_rom_rtl.vhd', 'w') #ferdig tabell
    fe = open('newton_int_divider_bipartit_reci_rom_ent.vhd', 'w') #grensesnitt til ferdig tabell
    fs = open('newton_int_divider_bipartit_reci_rom_statistikk.txt', 'w') #statistikk
    fp = open('newton_reci_table_pck.vhd','w') #pakke til oppfrses modell
    fb = open('newton_reci_table_bdy.vhd','w') #bodyen til pakken
    fsrp = open('newton_reci_table_sqr_pck.vhd','w') #pakka til kvaderatrot pakka
    fsrb = open('newton_reci_table_sqr_bdy.vhd','w') #bodyen til kvaderatrot pakka
except IOError:
    print 'Can\'t open file for writing.'
    sys.exit(0)

#lager vhdl filene
writeentetyfile(j)
writehedderrtl()
writePtable(j,k,u)   #her skjer ogs kalkuleringen av ptabellen.
writeNtable(k,u,j)   #N tabellen kalkuleres her.
writesubtraction(j,k,u)
writepck(bit, j, k, u)
writepckbdy(j, k, u)
writepcksqr(bit, j, k, u)
writepckbdysqr(j, k, u)


fe.close()
fi.close()
fs.close()
fp.close()
fb.close()
fsrp.close()
fsrb.close()

print "slutter\n"
