library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.numeric_std.all;
use work.divlib.all;

package body newton_reci_table is

  procedure init_look_up_table(tableP : inout look_up_tableP; tableN : inout look_up_tableN) is
    variable firstspread    : real;
    variable lastspread     : real;
    variable averagespread  : real;
    variable spread         : real;
    variable adjust         : real;
    variable p              : real;
    variable p_vec          : std_logic_vector(k+2 downto 0);
    variable firstdiff      : real;
    variable lastdiff       : real;
    variable n              : real;
    variable n_vec          : std_logic_vector(m downto 0);

    function resipmid(xh : integer; xm : integer; xl : integer) return real is
      variable temp : real;
    begin
      temp := real(xh)*real(2**(2*m+u))+real(xm)*real(2**m)+real(xl);
      return real(2**(3*m+u+1))/(real(2**(3*m+u+1))+temp+0.5);
    end resipmid;

  begin
    for ixh in 0 to 2**(m+1)-1 loop
      firstspread := resipmid(ixh, 0, 0)-resipmid(ixh, 0, 2**(m)-1);
      lastspread := resipmid(ixh, 2**(m+u)-1, 0)-resipmid(ixh, 2**(m+u)-1, 2**m-1);
      averagespread := (firstspread + lastspread)/2.0;
      for ixm in 0 to 2**(m+u)-1 loop
        spread := resipmid(ixh, ixm,0)-resipmid(ixh,ixm,2**m-1);
        adjust := (averagespread - spread)/2.0;
        p := (resipmid(ixh,ixm,0)+adjust)*real(2**(3*m+u+2))-0.5;
        p_vec := std_logic_vector(to_unsigned(integer(p),3*m+u+2));
        tableP(ixh*2**(m+u)+ixm+2**(2*m+u+1)) := p_vec(3*m+u downto 0);
      end loop;
    end loop;

    for jxh in 0 to 2**(m+1)-1 loop
      for jxl in 0 to 2**(m)-1 loop
        firstdiff := resipmid(jxh,0,0)-resipmid(jxh,0,jxl);
        lastdiff := resipmid(jxh,2**(m+u)-1,0) - resipmid(jxh,2**(m+u)-1,jxl);
        n := ((firstdiff+lastdiff)/2.0)*real(2**(3*m+u+2));
        n_vec := std_logic_vector(to_unsigned((integer(n)), m+1));
        tableN(jxh*2**(m)+jxl+2**(2*m+1)) := n_vec;
      end loop;
    end loop;
  end init_look_up_table;
end newton_reci_table;